@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
- * Copyright (C) 2009 Debarshi Ray <rishi@gnu.org>
+ * Copyright (C) 2009, 2010 Debarshi Ray <rishi@gnu.org>
  * Copyright (C) 2009 Santanu Sinha <santanu.sinha@gmail.com>
  *
  * Solang is free software: you can redistribute it and/or modify it
@@ -36,7 +36,7 @@
 #include "main-window.h"
 #include "photo.h"
 #include "photo-search-criteria.h"
-#include "thumbbuf-maker.h"
+#include "scale-action.h"
 
 namespace Solang
 {
@@ -46,6 +46,7 @@ static const guint higherZoomValue = 100;
 static const guint initialZoomValue
                        = lowerZoomValue
                          + (higherZoomValue - lowerZoomValue) / 2;
+static const guint stepZoomValue = 5;
 
 static const gint thumbnailRendererWidth = 168;
 static const gint thumbnailRendererHeight = 130;
@@ -69,7 +70,10 @@ BrowserRenderer::BrowserRenderer() throw() :
     sigc::trackable(),
     application_(NULL),
     iconFactory_(Gtk::IconFactory::create()),
-    actionGroup_(Gtk::ActionGroup::create()),
+    actionGroup_(Gtk::ActionGroup::create(
+                     Glib::ustring::compose("%1:%2",
+                                            __FILE__,
+                                            __LINE__))),
     uiID_(0),
     uiIDThumbnail_(0),
     dockItemName_("browser-dock-item"),
@@ -80,11 +84,12 @@ BrowserRenderer::BrowserRenderer() throw() :
     hBox_(false, 12),
     paginationBar_(),
     dummyLabel_(),
-    zoomer_(lowerZoomValue, higherZoomValue, initialZoomValue, 5.0),
     scrolledWindow_(),
     treeModelFilter_(),
     thumbnailView_(thumbnailRendererWidth, thumbnailRendererHeight),
+    zoomValue_(initialZoomValue),
     pageNum_(-1),
+    contextID_(),
     signalInitEnd_(),
     signalListStoreChangeBegin_(),
     signalListStoreChangeEnd_(),
@@ -142,42 +147,107 @@ BrowserRenderer::BrowserRenderer() throw() :
         sigc::mem_fun(*this,
                       &BrowserRenderer::on_action_view_slideshow));
 
-    actionGroup_->add(
-        zoomer_.action_zoom_in(),
-        Gtk::AccelKey("<control>plus"));
+    {
+        const Glib::RefPtr<ScaleAction> scale_action
+            = ScaleAction::create(
+                  "ActionViewBrowserZoom",
+                  _("Enlarge or shrink the thumbnails"),
+                  Gtk::Adjustment(initialZoomValue,
+                                  lowerZoomValue,
+                                  higherZoomValue,
+                                  stepZoomValue,
+                                  10 * stepZoomValue,
+                                  0));
 
-    actionGroup_->add(
-        zoomer_.action_zoom_out(),
-        Gtk::AccelKey("<control>minus"));
+        scale_action->reference();
 
-    actionGroup_->add(Gtk::Action::create(
-        "ActionViewEdit", Gtk::Stock::EDIT, _("_Edit"),
-        _("Edit picture")), Gtk::AccelKey("<control>e"),
-        sigc::mem_fun(*this, &BrowserRenderer::on_item_edit));
+        actionGroup_->add(scale_action);
+
+        scale_action->signal_value_changed().connect(
+            sigc::bind(
+                sigc::mem_fun(
+                    *this,
+                    &BrowserRenderer::on_action_view_zoom_changed),
+                scale_action));
+
+        actionGroup_->add(
+            Gtk::Action::create(
+                "ActionViewBrowserZoomIn", Gtk::Stock::ZOOM_IN,
+                _("_Zoom In"),
+                _("Enlarge the thumbnails")),
+            Gtk::AccelKey("<control>plus"),
+            sigc::bind(
+                sigc::mem_fun(
+                    *this,
+                    &BrowserRenderer::on_action_view_zoom_in),
+                scale_action));
+
+        actionGroup_->add(
+            Gtk::Action::create(
+                "ActionViewBrowserZoomOut", Gtk::Stock::ZOOM_OUT,
+                _("_Zoom Out"),
+                _("Shrink the thumbnails")),
+            Gtk::AccelKey("<control>minus"),
+            sigc::bind(
+                sigc::mem_fun(
+                    *this,
+                    &BrowserRenderer::on_action_view_zoom_out),
+                scale_action));
+    }
 
     actionGroup_->add(
         Gtk::Action::create(
-            "ActionMenuGo", _("_Go")));
+            "ActionBrowserGoMenu", _("_Go")));
+
+    {
+        ActionPtr action = Gtk::Action::create(
+            "ActionGoBrowserPreviousPage", Gtk::Stock::GO_BACK,
+            _("_Previous Page"),
+            _("Go to the previous page in the collection"));
+
+        action->property_short_label().set_value(_("Previous"));
+        action->property_is_important().set_value(true);
+        action->set_sensitive(false);
+        actionGroup_->add(
+            action, Gtk::AccelKey("<alt>Page_Up"),
+            sigc::mem_fun(*this,
+                          &BrowserRenderer::on_action_go_previous));
+    }
+
+    {
+        ActionPtr action = Gtk::Action::create(
+            "ActionGoBrowserNextPage", Gtk::Stock::GO_FORWARD,
+            _("_Next Page"),
+            _("Go to the next page in the collection"));
+
+        action->property_short_label().set_value(_("Next"));
+        action->property_is_important().set_value(true);
+        actionGroup_->add(
+            action, Gtk::AccelKey("<alt>Page_Down"),
+            sigc::mem_fun(*this,
+                          &BrowserRenderer::on_action_go_next));
+    }
+
+    {
+        ActionPtr action = Gtk::Action::create(
+            "ActionGoBrowserFirstPage", Gtk::Stock::GOTO_FIRST,
+            _("_First Page"),
+            _("Go to the first page in the collection"));
+
+        action->set_sensitive(false);
+        actionGroup_->add(
+            action, Gtk::AccelKey("<alt>Home"),
+            sigc::mem_fun(*this,
+                          &BrowserRenderer::on_action_go_first));
+    }
 
     actionGroup_->add(
-        paginationBar_.action_previous(),
-        Gtk::AccelKey("<alt>Page_Up"));
-
-    actionGroup_->add(
-        paginationBar_.action_next(),
-        Gtk::AccelKey("<alt>Page_Down"));
-
-    actionGroup_->add(
-        paginationBar_.action_first(),
-        Gtk::AccelKey("<alt>Home"));
-
-    actionGroup_->add(
-        paginationBar_.action_last(),
-        Gtk::AccelKey("<alt>End"));
-
-    hBox_.pack_start(paginationBar_, Gtk::PACK_SHRINK, 0);
-    hBox_.pack_start(dummyLabel_, Gtk::PACK_EXPAND_PADDING, 0);
-    hBox_.pack_start(zoomer_, Gtk::PACK_SHRINK, 0);
+        Gtk::Action::create(
+            "ActionGoBrowserLastPage", Gtk::Stock::GOTO_LAST,
+            _("_Last Page"),
+            _("Go to the last page in the collection")),
+        Gtk::AccelKey("<alt>End"),
+        sigc::mem_fun(*this, &BrowserRenderer::on_action_go_last));
 
     vBox_.pack_start(hBox_, Gtk::PACK_SHRINK, 0);
 
@@ -195,14 +265,15 @@ BrowserRenderer::BrowserRenderer() throw() :
     gtk_container_add(GTK_CONTAINER(dockItem_),
                       GTK_WIDGET(vBox_.gobj()));
 
+    Gtk::Widget * const dock_item = Glib::wrap(dockItem_, false);
+    dock_item->signal_parent_changed().connect(
+        sigc::mem_fun(*this,
+                      &BrowserRenderer::on_dock_item_parent_changed));
+
     paginationBar_.limits_changed().connect(
         sigc::mem_fun(*this,
                       &BrowserRenderer::on_limits_changed));
 
-    zoomer_.get_scale().signal_value_changed().connect(
-        sigc::mem_fun(*this,
-                      &BrowserRenderer::on_signal_value_changed));
-
     const UIManagerPtr & ui_manager = thumbnailView_.get_ui_manager();
     uiIDThumbnail_ = ui_manager->add_ui_from_file(uiFileThumbnail);
 
@@ -239,11 +310,13 @@ BrowserRenderer::init(Application & application) throw()
                                        &BrowserRenderer::on_visible));
     thumbnailView_.set_model(treeModelFilter_);
 
-    Engine & engine = application.get_engine();
-
     MainWindow & main_window = application.get_main_window();
     main_window.add_dock_object_center(GDL_DOCK_OBJECT(dockItem_));
 
+    Gtk::Statusbar & status_bar = main_window.get_status_bar();
+    contextID_ = status_bar.get_context_id(
+                     Glib::ustring::compose("%1", G_STRLOC));
+
     const UIManagerPtr & ui_manager = main_window.get_ui_manager();
 
     uiID_ = ui_manager->add_ui_from_file(uiFile);
@@ -254,13 +327,6 @@ BrowserRenderer::init(Application & application) throw()
 
     ui_manager->insert_action_group(actionGroup_);
 
-    Glib::ThreadPool & thread_pool = application.get_thread_pool();
-    PhotoSearchCriteriaList criterion;
-
-    thread_pool.push(sigc::bind(
-        sigc::mem_fun1<const PhotoSearchCriteriaList &>(
-            engine, &Engine::show), criterion));
-
     signalInitEnd_
         = application.init_end().connect(
               sigc::mem_fun(*this,
@@ -338,13 +404,11 @@ BrowserRenderer::clear_thumbnails() throw()
         BrowserModelColumnRecord model_column_record;
         Gtk::TreeModel::Row row = *iter;
 
-        row[model_column_record.get_column_pixbuf()] = PixbufPtr(0);
         PhotoPtr photo = row[model_column_record.get_column_photo()];
         if( !photo->get_has_unsaved_data() )
         {
             photo->set_thumbnail_buffer( PixbufPtr(0) );
         }
-        row[model_column_record.get_column_tag_name()] = "";
 
         while (true == Gtk::Main::events_pending())
         {
@@ -353,98 +417,6 @@ BrowserRenderer::clear_thumbnails() throw()
     }
 }
 
-void
-BrowserRenderer::generate_thumbnails() throw()
-{
-    std::ostringstream fsout;
-    fsout << paginationBar_.get_lower_limit();
-
-    const Gtk::TreeModel::Path first_path(fsout.str());
-
-    if (true == first_path.empty())
-    {
-        return;
-    }
-
-    const Gtk::TreeModel::iterator first_iter
-        = treeModelFilter_->get_model()->get_iter(first_path);
-
-    if (false == first_iter)
-    {
-        return;
-    }
-
-    std::ostringstream lsout;
-    lsout << paginationBar_.get_upper_limit() - 1;
-
-    const Gtk::TreeModel::Path last_path(lsout.str());
-
-    if (true == last_path.empty())
-    {
-        return;
-    }
-
-    const Gtk::TreeModel::iterator last_iter
-        = treeModelFilter_->get_model()->get_iter(last_path);
-
-    if (false == last_iter)
-    {
-        return;
-    }
-
-    const guint zoom_value = zoomer_.get_scale().get_value();
-    const guint thumbnail_width
-                    = static_cast<guint>(
-                          ratioWidth
-                          * static_cast<double>(zoom_value));
-    const guint thumbnail_height
-                    = static_cast<guint>(
-                          ratioHeight
-                          * static_cast<double>(zoom_value));
-
-    thumbnailView_.set_thumbnail_width(thumbnail_width + 12);
-    thumbnailView_.set_thumbnail_height(thumbnail_height + 12);
-
-    Gtk::TreeModel::iterator iter;
-
-    // operator<= is not defined.
-    for (iter = first_iter; ; iter++)
-    {
-        BrowserModelColumnRecord model_column_record;
-        Gtk::TreeModel::Row row = *iter;
-
-        const PhotoPtr & photo
-            = row[model_column_record.get_column_photo()];
-        const PixbufPtr & pixbuf
-            = row[model_column_record.get_column_pixbuf()];
-
-        if (0 == photo->get_thumbnail_buffer() )
-        {
-            ThumbbufMaker thumbbuf_maker(thumbnail_width,
-                                         thumbnail_height);
-
-            photo->set_thumbnail_buffer( thumbbuf_maker(photo) );
-
-            row[model_column_record.get_column_pixbuf()]
-                = photo->get_thumbnail_buffer();
-
-            row[model_column_record.get_column_tag_name()]
-                = photo->get_exif_data().get_picture_taken_time();
-        }
-        else
-        {
-            row[model_column_record.get_column_pixbuf()]
-                = photo->get_thumbnail_buffer();
-        }
-
-        // operator<= is not defined.
-        if (last_iter == iter)
-        {
-            break;
-        }
-    }
-}
-
 PhotoList
 BrowserRenderer::get_current_selection() throw()
 {
@@ -479,6 +451,30 @@ BrowserRenderer::on_action_add_to_export_queue() throw()
     export_queue_inserter();
 }
 
+void
+BrowserRenderer::on_action_go_previous() throw()
+{
+    paginationBar_.go_previous();
+}
+
+void
+BrowserRenderer::on_action_go_next() throw()
+{
+    paginationBar_.go_next();
+}
+
+void
+BrowserRenderer::on_action_go_first() throw()
+{
+    paginationBar_.go_first();
+}
+
+void
+BrowserRenderer::on_action_go_last() throw()
+{
+    paginationBar_.go_last();
+}
+
 void
 BrowserRenderer::on_action_view_slideshow() throw()
 {
@@ -514,19 +510,36 @@ BrowserRenderer::on_action_view_slideshow() throw()
     BrowserModelColumnRecord model_column_record;
     const PhotoPtr photo = row[model_column_record.get_column_photo()];
 
-    slideshow_renderer->render(photo);
     slideshow_renderer->present();
+    slideshow_renderer->render(photo);
 }
 
 void
 BrowserRenderer::on_init_end(Application & application) throw()
 {
-    MainWindow & main_window = application.get_main_window();
-    Gtk::Notebook * notebook = main_window.get_notebook_center();
+    signalInitEnd_.disconnect();
+}
+
+void
+BrowserRenderer::on_selection_changed() throw()
+{
+    application_->get_engine().selection_changed().emit();
+}
+
+void
+BrowserRenderer::on_dock_item_parent_changed(
+                     Gtk::Widget * previous_parent) throw()
+{
+    signalSwitchPage_.disconnect();
+
+    Gtk::Widget * const parent = Glib::wrap(gtk_widget_get_parent(
+                                                dockItem_), false);
+    Gtk::Notebook * const notebook = dynamic_cast<Gtk::Notebook *>(
+                                         parent);
 
     if (0 == notebook)
     {
-        g_warning("0 == notebook");
+        pageNum_ = 0;
         return;
     }
 
@@ -536,14 +549,6 @@ BrowserRenderer::on_init_end(Application & application) throw()
         = notebook->signal_switch_page().connect(
               sigc::mem_fun(*this,
                             &BrowserRenderer::on_switch_page));
-
-    signalInitEnd_.disconnect();
-}
-
-void
-BrowserRenderer::on_selection_changed() throw()
-{
-    application_->get_engine().selection_changed().emit();
 }
 
 void
@@ -571,34 +576,46 @@ BrowserRenderer::on_item_activated(const Gtk::TreeModel::Path & path)
     BrowserModelColumnRecord model_column_record;
     const PhotoPtr photo = row[model_column_record.get_column_photo()];
 
-    enlarged_renderer->render(photo);
     enlarged_renderer->present();
-}
-
-void
-BrowserRenderer::on_item_edit() throw()
-{
-    RendererRegistry & renderer_registry
-        = application_->get_renderer_registry();
-    const IRendererPtr editor_renderer
-        = renderer_registry.select<EditorRenderer>();
-
-    if (0 == editor_renderer)
-    {
-        return;
-    }
-
-    const PhotoList photos = thumbnailView_.get_selected_photos();
-
-    editor_renderer->render(photos);
-    editor_renderer->present();
+    enlarged_renderer->render(photo);
 }
 
 void
 BrowserRenderer::on_limits_changed() throw()
 {
-    generate_thumbnails();
+    const guint total = paginationBar_.get_total();
+    const guint lower_limit = paginationBar_.get_lower_limit();
+    const guint upper_limit = paginationBar_.get_upper_limit();
+
+    {
+        const ActionPtr action = actionGroup_->get_action(
+                                     "ActionGoBrowserPreviousPage");
+        action->set_sensitive((0 < lower_limit) ? true : false);
+    }
+
+    {
+        const ActionPtr action = actionGroup_->get_action(
+                                     "ActionGoBrowserFirstPage");
+        action->set_sensitive((0 < lower_limit) ? true : false);
+    }
+
+    {
+        const ActionPtr action = actionGroup_->get_action(
+                                     "ActionGoBrowserNextPage");
+        action->set_sensitive((total > upper_limit) ? true : false);
+    }
+
+    {
+        const ActionPtr action = actionGroup_->get_action(
+                                     "ActionGoBrowserLastPage");
+        action->set_sensitive((total > upper_limit) ? true : false);
+    }
+
+    set_thumbnail_size();
     treeModelFilter_->refilter();
+
+    status_pop();
+    status_push();
 }
 
 void
@@ -630,8 +647,11 @@ BrowserRenderer::on_list_store_change_end(Application & application)
 }
 
 void
-BrowserRenderer::on_signal_value_changed() throw()
+BrowserRenderer::on_action_view_zoom_changed(
+                     const ScaleActionPtr & scale_action) throw()
 {
+    zoomValue_ = scale_action->get_value();
+
     static sigc::connection connection;
 
     connection.disconnect();
@@ -643,6 +663,20 @@ BrowserRenderer::on_signal_value_changed() throw()
               1, Glib::PRIORITY_DEFAULT);
 }
 
+void
+BrowserRenderer::on_action_view_zoom_in(
+                     const ScaleActionPtr & scale_action) throw()
+{
+    scale_action->set_value(zoomValue_ + stepZoomValue);
+}
+
+void
+BrowserRenderer::on_action_view_zoom_out(
+                     const ScaleActionPtr & scale_action) throw()
+{
+    scale_action->set_value(zoomValue_ - stepZoomValue);
+}
+
 void
 BrowserRenderer::on_switch_page(GtkNotebookPage * notebook_page,
                                 guint page_num) throw()
@@ -689,6 +723,7 @@ BrowserRenderer::on_switch_page(GtkNotebookPage * notebook_page,
         }
 
         signalSelectionChanged_.unblock();
+        status_push();
 
         if (0 == uiID_)
         {
@@ -700,7 +735,6 @@ BrowserRenderer::on_switch_page(GtkNotebookPage * notebook_page,
 
             ui_manager->insert_action_group(actionGroup_);
         }
-        reload();
     }
     else
     {
@@ -712,6 +746,7 @@ BrowserRenderer::on_switch_page(GtkNotebookPage * notebook_page,
         }
 
         signalSelectionChanged_.block();
+        status_pop();
     }
 }
 
@@ -732,9 +767,48 @@ BrowserRenderer::on_visible(
 void
 BrowserRenderer::reload() throw()
 {
+    set_thumbnail_size();
     clear_thumbnails();
-    generate_thumbnails();
     treeModelFilter_->refilter();
 }
 
+void
+BrowserRenderer::set_thumbnail_size() throw()
+{
+    const guint thumbnail_width
+                    = static_cast<guint>(
+                          ratioWidth
+                          * static_cast<double>(zoomValue_));
+    const guint thumbnail_height
+                    = static_cast<guint>(
+                          ratioHeight
+                          * static_cast<double>(zoomValue_));
+
+    thumbnailView_.set_thumbnail_width(thumbnail_width + 6);
+    thumbnailView_.set_thumbnail_height(thumbnail_height + 6);
+}
+
+void
+BrowserRenderer::status_pop() throw()
+{
+    MainWindow & main_window = application_->get_main_window();
+    Gtk::Statusbar & status_bar = main_window.get_status_bar();
+    status_bar.pop(contextID_);
+}
+
+void
+BrowserRenderer::status_push() throw()
+{
+    const guint total = paginationBar_.get_total();
+    const guint lower_limit = paginationBar_.get_lower_limit();
+    const guint upper_limit = paginationBar_.get_upper_limit();
+
+    MainWindow & main_window = application_->get_main_window();
+    Gtk::Statusbar & status_bar = main_window.get_status_bar();
+
+    std::ostringstream sout;
+    sout << lower_limit << " - " << upper_limit << " of " << total;
+    status_bar.push(sout.str(), contextID_);
+}
+
 } // namespace Solang
